////////////////////
// Initialization //
////////////////////

bridge_path   = '/home/collette/Aster/Bridge/';
aster_path    = '/opt/stow/aster/';

as_run        = aster_path + 'ASTK/ASTK_SERV/bin/as_run';
gmsh          = aster_path + 'outils/gmsh';

tpl_file_name = 'bridge_geo_template.txt';
out_file_name = 'bridge.geo';
nb_snapshots  = 10;

var_names     = ['x11','x12','y11','y12','z11','z12', ...
                 'x21','x22','y21','y22','z21','z22', ...
                 'x31','x32','y31','y32','z31','z32', ...
                 'x41','x42','y41','y42','z41','z42', ...
                 'x51','x52','y51','y52','z51','z52', ...
                 'x61','x62','y61','y62','z61','z62', ...
                 'x71','x72','y71','y72','z71','z72'];

// Initial structure
                 
x0 = [1.0, 1.0, 2.0, 2.0, 0.0, 1.0, ...
      2.0, 2.0, 2.0, 2.0, 0.0, 1.0, ...
      3.0, 3.0, 2.0, 2.0, 0.0, 1.0, ... 
      4.0, 4.0, 2.0, 2.0, 0.0, 1.0, ...
      5.0, 5.0, 2.0, 2.0, 0.0, 1.0, ... 
      6.0, 6.0, 2.0, 2.0, 0.0, 1.0, ...
      7.0, 7.0, 2.0, 2.0, 0.0, 1.0]'; 

// Boundaries for each coordinates

x_min = [0.0, 0.0, 1.1, 1.1, -1.0, -1.0, ...
         1.0, 1.0, 1.1, 1.1, -1.0, -1.0, ...
         2.0, 2.0, 1.1, 1.1, -1.0, -1.0, ... 
         3.0, 3.0, 1.1, 1.1, -1.0, -1.0, ...
         4.0, 4.0, 1.1, 1.1, -1.0, -1.0, ... 
         5.0, 5.0, 1.1, 1.1, -1.0, -1.0, ...
         6.0, 6.0, 1.1, 1.1, -1.0, -1.0]'; 

x_max = [2.0, 2.0, 10.0, 10.0, 2.0, 2.0, ...
         3.0, 3.0, 10.0, 10.0, 2.0, 2.0, ...
         4.0, 4.0, 10.0, 10.0, 2.0, 2.0, ... 
         5.0, 5.0, 10.0, 10.0, 2.0, 2.0, ...
         6.0, 6.0, 10.0, 10.0, 2.0, 2.0, ... 
         7.0, 7.0, 10.0, 10.0, 2.0, 2.0, ...
         8.0, 8.0, 10.0, 10.0, 2.0, 2.0]'; 

x_snap = zeros(length(x_max),nb_snapshots);
for i=1:nb_snapshots
  printf('Computing snapshot %d / %d', i, nb_snapshots);

  x_snap(:,i) = (x_max - x_min).*rand(x_max) + x_min;

  var_values = x_snap(:,i);
  template_replace(tpl_file_name, var_names, var_values, out_file_name);

  [rep,stat,err]=unix_g(gmsh + ' -3 ' + bridge_path + out_file_name + ' -o ' + bridge_path + 'bridge.msh');
  if (stat~=0) then
    printf('rep = '); disp(rep);
    printf('err = '); disp(err);
  end

  timer(); // Timer initialization
  [rep,stat,err]=unix_g(as_run + ' ' + bridge_path + 'bridge.export');
  t2 = timer();

  if (stat~=0) then
    printf('rep = '); disp(rep);
    printf('err = '); disp(err);
  end

  fid   = mopen(bridge_path + 'Bridge_Result.dat','r');
  Lines = mgetl(fid);
  mclose(fid);
  Lines = evstr(Lines);
  dx_snap(:,i) = Lines(:,2);
  dy_snap(:,i) = Lines(:,3);

  printf(' - %f sec dx = %f - dy = %f\n', t2, dx_snap(:,i), dy_snap(:,i));
end

// Compute linear regression models
for i=1:size(dx_snap,1)
  dx_model(:,i) = x_snap' \ dx_snap(i,:)';
  dy_model(:,i) = x_snap' \ dy_snap(i,:)';
end

////////////////////////////
// The objective function //
////////////////////////////

// Main objective function

function [f] = fobj(x)
  f = 0;
  for i=1:size(dx_model,2)
    f = f + (dx_model(:,i)' * x)^2;
    f = f + (dy_model(:,i)' * x)^2;
  end
  global Eval_Fobj;
  Eval_Fobj = Eval_Fobj + 1;
  printf('Function evaluation %d: %f - %f sec\n', Eval_Fobj, f, t2);
endfunction

// A wrapper fobj for optim

function [f,g,ind] = fobj_optim(x,ind)
  global Iteration;
  Iteration = Iteration + 1;
  printf('Iteration %d starting\n', Iteration);
  f = fobj(x);
  g = derivative(fobj,x,h=1e-2);
  printf('Iteration %d finished\n', Iteration);
endfunction

///////////////
// Main part //
///////////////

t_start = getdate();
[f_opt, x_opt] = optim(fobj_optim,'b',x_min,x_max,x0,'gc','ar',10,10);
t_end = getdate();

var_values = x_opt;

template_replace(tpl_file_name, var_names, var_values, 'bridge_opt_pod.geo');
